@@ -21,26 +21,54 @@
 
 
 
-import hsegLibrary
+from pyspatiotemporalgeom.utilities import hsegLibrary
 import math
 import collections
-import convexHull
-import region
+from pyspatiotemporalgeom.utilities import convexHull
+import pyspatiotemporalgeom.region as region
 
 def dot( u, v ):
-        # returns dot product of vectors u and v
+        ''' 
+        returns dot product of vectors u and v
+        
+        u and v are 3-tuples representing a 3D point: ``(x,y,z)``
+
+        This function is used to compute triangle/triangle intersections
+
+        Returns a number
+        '''
         return u[0]*v[0] + u[1]*v[1] + u[2]*v[2]
 
 def crossProduct( v, u ):
-        #returns cross product of u and v
+        '''
+        returns cross product of u and v
+
+        u and v are 3-tuples representing a 3D point: ``(x,y,z)``
+
+        This function is used to compute triangle/triangle intersections
+
+        Returns a 3-tuple of numbers (a 3D point): ``(x,y,z)``
+        '''
         return( (v[1]*u[2]-u[1]*v[2]), (-v[0]*u[2]+u[0]*v[2]),(v[0]*u[1]-u[0]*v[1]) )
 
 
 def triangleTriangleIntersection( tri1, tri2 ):
         '''
-        triangles always have points [0] and [1] from the same slice.
+        Input:  two triangles of the form: ``((x1,y1,z1),(x2,y2,z2),(x3,y3,z3))``
+
+        triangles always have points [0] and [1] from the same slice. In other words, for the representation listed above, z1 = z2 ALWAYS. 
+        
         So ... always make segs with [0][2] and [1][2] for tri tri intersection
-        We are only interested in intersections in the interior of triangles
+        
+        We are only interested in intersections in the interior of triangles.
+
+        RETURNS:
+
+          ``True`` if the triangles intersect in their interior
+
+          ``False`` if the trinagle's interiors do not intersect.  
+
+          Triangles with coincident edges but disjoint interiors return ``False``
         '''
         t1s1 = (tri1[0], tri1[2] )
         t1s2 = (tri1[1], tri1[2] )
@@ -62,28 +90,46 @@ def triangleTriangleIntersection( tri1, tri2 ):
 
 def triangleRayIntersection( ray, tri ):
         '''
-        ray is actually a seg in our case.  To conver this algorithm to use rays, change the line:
-            if r <= 0.0 or r >= 1.0:
-          to:
-            if r <= 0.0:
+        
+        Test if a triangle and a ray intersect.  A ray is actually a seg in our case. 
+        
+        To conver this algorithm to use rays, change the line:
+            
+        ``if r <= 0.0 or r >= 1.0:``
+        
+        to:
+        
+        ``if r <= 0.0:``
+
         We will always pass segs, since triangles will end at slice boundaries
-        tri is a triangle a tuple of 3 3D points.  Triangle vertices in arbitrary order
-           Returns:
-           -1: triangle is degenerate (seg or a point)
-           0: disjoint
-           1: intersect in unique point
-           2: in the same plane.  
-           This function is modeled from triangle ray intersection algorithm at http://geomalgorithms.com/body_a06-_intersect-2.html#intersect3D_RayTriangle(),
-           although this algorithm is different to handle specific intersection cases differently.  Comments are included that identify
-           portions specific to our intersection needs
-           The code at http://geomalgorithms.com/body_a06-_intersect-2.html#intersect3D_RayTriangle() has the following copyright:
-                 //Copyright 2001 softSurfer, 2012 Dan Sunday
-                 // This code may be freely used and modified for any purpose
-                 // providing that this copyright notice is included with it.
-                 // SoftSurfer makes no warranty for this code, and cannot be held
-                 // liable for any real or imagined damage resulting from its use.
-                 // Users of this code must verify correctness for their application.
-           '''
+        
+        tri is a triangle a tuple of 3, 3D points.  Triangle vertices in arbitrary order
+        
+        Returns:
+        
+        -1: triangle is degenerate (seg or a point)
+        
+        0: disjoint
+        
+        1: intersect in unique point
+        
+        2: intersects in the same plane. (the intersection is a line segment)  
+        
+        This function is modeled from triangle ray intersection algorithm at http://geomalgorithms.com/body_a06-_intersect-2.html#intersect3D_RayTriangle(),
+         although this algorithm is different to handle specific intersection cases differently.  Comments are included that identify
+        portions specific to our intersection needs
+        The code at http://geomalgorithms.com/body_a06-_intersect-2.html#intersect3D_RayTriangle() has the following copyright:
+        
+        .. code-block:: c
+
+          //Copyright 2001 softSurfer, 2012 Dan Sunday
+          // This code may be freely used and modified for any purpose
+          // providing that this copyright notice is included with it.
+          // SoftSurfer makes no warranty for this code, and cannot be held
+          // liable for any real or imagined damage resulting from its use.
+          // Users of this code must verify correctness for their application.
+        
+        '''
         # easy test.  if the ray is a tri seg, return not intersecting
         # we don't consider a ray that is identical to a triangle edge to be intersecting
         tsegs = ((tri[0], tri[1] ), (tri[1], tri[0] ), (tri[0], tri[2] ), (tri[2], tri[0] ), (tri[1], tri[2] ), (tri[2], tri[1] ))
@@ -139,13 +185,24 @@ def triangleRayIntersection( ray, tri ):
 
 def prepRegionForInterpolation( hsegs ):
         '''
-        Assumes a valid region in hseg order returned from region.getRandomIntegerRegion()
-        Assumes each cycle in the region has its own unique label, and -1 for exterior labels
-        Returns the region with cycles labeled for interoplation
-        Returns the convex hull points in CCW sequence
-        Returns the cycle mapping.
-        Returns list of connected cycles
-        Returns a dict that maps a point to a list of cycles connected to that point
+        This function is used to prepare regions for interpolation.
+        
+        It is expected that this function is only useful for the ``interpolateRegions()`` function.
+
+        First, connected cycles in a region are identified (cycles that touch).  Then connected cycles are relabeled so the interior label is identical for each group of connected cycles.  Then vertical connector segments are added.  The point sequence of an outercycle walk is computed.  The convex hull of that sequence is computed. Finally, labels are assigned for concavity mappings.
+
+        ASSUMPTIONS:
+        
+        1. ``hsegs`` is a valid region in hseg order returned from region.createRegionFromSegs() or any of the region creation functions.
+        2. Each cycle in the region has its own unique label, and -1 for exterior labels.
+
+        RETURNS teh following as a tuple:
+
+        1. Returns the region with cycles labeled for interoplation
+        2. Returns the convex hull points in CCW sequence
+        3. Returns the cycle mapping.
+        4. Returns list of connected cycles
+        5. Returns a dict that maps a point to a list of cycles connected to that point
         '''
         # find which cycles are connected
         cycleConnectionMapList = hsegLibrary.getConnectedCycleMapping( hsegs )
@@ -170,6 +227,14 @@ def prepRegionForInterpolation( hsegs ):
 
 
 def angleFromVertical( seg ):
+        '''
+        A utility to compute the angle a segment forms with a vertical line emanating downwards from the least end point of the segment.
+
+        Input: A line segment ((x1,y1),(x2,y2))
+
+        Returns: an angle in degrees
+
+        '''
         PI = math.pi
         p = seg[0]
         q = seg[1]
@@ -212,14 +277,28 @@ def angleFromVertical( seg ):
 
 def writeTrisToFile( triTup, fileName ):
         '''
+        Write triangles to a file.
+
+        This was used during debugging.
+
         expects an iterable containing iterables ( a list of lists).
+        
         Each iterable contained in the first contains triangles.  
+        
         A trianlge is a 3 tuple of points (p0, p1, p2) == ( (x0,y0,z0), (x1,y1,z1), (x2,y2,z2) ) or 3d points
+        
         A point is a 3 tuple.  
+        
+        Input: 
+
         triTup = (listofTris1, listofTris2,...) 
-            where listofTrisX = [t1, t2, t3,...]
-            where tX = (p1,p2,p3) 
-            where pX=(x0, y0, z0,)
+        
+        where listofTrisX = [t1, t2, t3,...]
+        
+        where tX = (p1,p2,p3) 
+        
+        where pX=(x0, y0, z0,)
+        
         Prints the contents to a file named fileName.  will clobber that file
         '''
         f = open( fileName, 'w')
@@ -238,11 +317,15 @@ def writeTrisToFile( triTup, fileName ):
 
 def appendToAnimationFile( triList, fileObject, numFrames = 100 ):
         '''
-        Writes snapshots to a file.  file object should already be open and writeable
-        triList is an iterable of triangles
-        a triangle is a 3 tuple containind 3 3d points:  ( (x0,y0,z0), (x1,y1,z1), (x2,y2,z2) ) 
+        For an triangle list defining a moving region over a single time interval (for instance, one element of the tuple returned by ``interpolateRegions()``), write snapshots of that region as it moves over the time interval.  In essence, create files that show an animation of the moving region. 
 
-        Writes segs to the text file for a snapshot, then appends a newline.
+        Input:
+
+        triList: A list of triangles in the usual format: ``( (x0,y0,z0), (x1,y1,z1), (x2,y2,z2) ) ``
+
+        fileObject: An open file object that the snapshot will be written to.
+
+        numFrames: The number of snapshots to generate (the number of frames in thr resulting animation if one were to create a movie out of them).
         '''
         if triList == None:
                 return
@@ -280,6 +363,22 @@ def appendToAnimationFile( triList, fileObject, numFrames = 100 ):
         
 
 def isColinearAndOverlapping( Lseg, Rseg, tolerance = .000001 ):
+        '''
+        check if two segments are colinear within a tolerance and overlapping
+
+        Input:
+        
+        Lseg and Rseg: two line segments
+
+        Tolerance: A tolerance value for colinearity
+
+        Returns:
+
+          ``True`` if the segments are colinear within tolerance AND overlapping
+
+          ``False`` otherwise
+        '''
+
         L = (Lseg[0], Lseg[1])
         R = (Rseg[0], Rseg[1])
         
@@ -317,9 +416,17 @@ def isColinearAndOverlapping( Lseg, Rseg, tolerance = .000001 ):
         return False
 
 def colinearValue( p1,p2,p3):
-        # will return a 0 if p1, p2,p3 are exactly colinear
-        # will return positive value if it forms a left turn
-        # will return neg value if it forms a right turn
+        '''
+
+        p1,p2, and p3 are points: (x,y)
+
+        will return a 0 if p1, p2,p3 are exactly colinear
+        
+        will return positive value if it forms a left turn
+        
+        will return neg value if it forms a right turn
+        
+        '''
         p1x = p1[0]
         p1y = p1[1]
         p2x = p2[0]
@@ -330,6 +437,10 @@ def colinearValue( p1,p2,p3):
         
         
 def writeHsegsToTextFile( fileName, hsegs ):
+        '''
+        Used in debugging.  Open a file named *filename* and clobber it if it exists.  Then write all the halfsegments in the list ``hsegs`` to that file.  1 hseg per line, each point and label seperated by a space.
+        '''
+
         f = open( fileName, 'w' )
         for h in hsegs:
                 if hsegLibrary.isLeft( h ):
@@ -345,10 +456,29 @@ def writeSegsToTextFile( fileName, segs ):
 
 def interpolate(r1,r2, startTime, endTime, noTriIntersectionChecks = False ):
         '''
-        creates motion triangles describing the movement of segs across an interval to interpolate between regions.
-        returns 3 list, corresponding to the maximum of 3 time intervals that may be produced.  If a list is empty,
-        that time interval was not used.  The middle interval always has something in it.
-        If noTriIntersectionChecks is set to True, the first and last intervals wil always be empty.
+        This is where the magic happens.  Create an interpolation between two well-formed regions over a time interval (defined by ``startTime`` and ``endTime``) such that at every instant within that time interval, the region resulting from the interpolation at that instant conforms to the type definition of complex regions as defined in [1].  Note that the various region generators and region creation functions int he region.py file create well formed regions according to [1].  In otherwords, the moving region resulting from this function conforms to the type definition of moving regions in [2].
+
+        This function is an extension of the algorithm in [3] to handle both simple (1 simple cycle with no holes) regions and complex regions.
+
+        
+        [1] Markus Schneider and Thomas Behr. 2006. Topological relationships between complex spatial objects. ACM Trans. Database Syst. 31, 1 (March 2006), 39-81. DOI=10.1145/1132863.1132865 http://doi.acm.org/10.1145/1132863.1132865
+
+        [2] Ralf Hartmut Guting, Michael H. Bohlen, Martin Erwig, Christian S. Jensen, Nikos A. Lorentzos, Markus Schneider, and Michalis Vazirgiannis. 2000. A foundation for representing and querying moving objects. ACM Trans. Database Syst. 25, 1 (March 2000), 1-42. DOI=10.1145/352958.352963 http://doi.acm.org/10.1145/352958.352963
+
+        [3] Mark McKenney and James Webb. 2010. Extracting moving regions from spatial data. In Proceedings of the 18th SIGSPATIAL International Conference on Advances in Geographic Information Systems (GIS '10). ACM, New York, NY, USA, 438-441. DOI=10.1145/1869790.1869856 http://doi.acm.org/10.1145/1869790.1869856
+
+        Input:
+          
+        1. r1, r2: two well formed regions represented as lists of hlafsegments.  Any of the region creation functions in region.py will do.
+        2. startTime, endTime:  two numbers defining a time interval.  These numbers are used as the 3D dimension when extrapolating into 3D space.
+        3. noTriIntersectionChecks. See paper [3].  The algorithm first creates an interpolation between the input regions.  It is possible that the interpolation will result in a self-intersecting region at some point.  The triangle/triangle intersection test is then performed.  This test is very computationally intensive (especially for python) and can take a LONG time to compute.  If you pass a ``True`` for this argument, the tri/tri intersection test is skipped, and the interpolation returned AS-IS (possibly with self-intersections).  This makes the algorithm :math:`O(n \lg n)` instead of :math:`O(n^2)`.
+
+        Output:
+
+        A 3-tuple.  See [3].  The algorithm will create at MOST, 3 interval regions to describe the interpolation of r1 to r2 over the defined time interval.  Not all 3 interval regions are always required, so 1 or 2 of the values in the tuple may be ``None``, but a 3 tuple is ALWAYS returned.  If the ``noTriIntersectionChecks`` argument is set to ``True``, or the original interpolation succeeds, then the return value will look like this: ``(None, triList, None)``.
+
+        Any non-``None`` value in the return tuple will be a list of trinagles describing the movinement of line segments in r1 as they travel across the defined interval to r2 (between intermediate states of r1 and r2 if necessary).
+        
         '''
         r1, r1Hull, r1LabelMapping, r1ConnCycLists, r1PoiToCycLabelDict  = prepRegionForInterpolation( r1 )
         r2, r2Hull, r2LabelMapping, r2ConnCycLists, r2PoiToCycLabelDict  = prepRegionForInterpolation( r2 )
@@ -361,7 +491,8 @@ def interpolate(r1,r2, startTime, endTime, noTriIntersectionChecks = False ):
         printList = [r1HullTris+r2HullTris]
         printList.append( [t for clist in r1ConcTris for t in clist] )
         printList.append(  [t for clist in r2ConcTris for t in clist] )
-        writeTrisToFile( printList, 'debug_tri3d_initial.txt')
+        # DEBUG -- uncomment the following
+        #writeTrisToFile( printList, 'debug_tri3d_initial.txt')
         
         if noTriIntersectionChecks:
                 # create the return list
@@ -449,8 +580,9 @@ def interpolate(r1,r2, startTime, endTime, noTriIntersectionChecks = False ):
                         concSegs.append( r1ConcHullSeg[i] )
                 # triangulate it
                 hsegs = region.createRegionFromSegs( concSegs )
-                writeHsegsToTextFile( 'debug_tri_hsegs_evap'+str(i)+'.txt', hsegs )
-                writeSegsToTextFile( 'debug_tri_segs_evap'+str(i)+'.txt', concSegs )
+                # DEBUG -- uncomment the following 2 lines
+                #writeHsegsToTextFile( 'debug_tri_hsegs_evap'+str(i)+'.txt', hsegs )
+                #writeSegsToTextFile( 'debug_tri_segs_evap'+str(i)+'.txt', concSegs )
                 theTris =  hsegLibrary.triangulate( hsegs )
                 theTris = [ t+(i,) for t in theTris ]
                 evapTris.extend( theTris )
@@ -486,8 +618,9 @@ def interpolate(r1,r2, startTime, endTime, noTriIntersectionChecks = False ):
                         concSegs.append( r2ConcHullSeg[i] )
                 # triangulate it
                 hsegs = region.createRegionFromSegs( concSegs )
-                writeHsegsToTextFile( 'debug_tri_hsegs_cond'+str(i)+'.txt', hsegs )
-                writeSegsToTextFile( 'debug_tri_segs_cond'+str(i)+'.txt', concSegs )
+                # DEBUG -- uncomment the following 2 line
+                #writeHsegsToTextFile( 'debug_tri_hsegs_cond'+str(i)+'.txt', hsegs )
+                #writeSegsToTextFile( 'debug_tri_segs_cond'+str(i)+'.txt', concSegs )
                 theTris = hsegLibrary.triangulate( hsegs )
                 theTris = [ t+(i,) for t in theTris ]
                 condTris.extend( theTris )
@@ -589,6 +722,9 @@ def interpolate(r1,r2, startTime, endTime, noTriIntersectionChecks = False ):
 
 
 def checkBoundaryExistence( tris ):
+        ''' Becuase of the way the interpolation algorithm works, some segments do not actually exist at the temporal extrema of the interval.  This function appends a boolean to each triangle indicating if the segment portion of the trinagle should be printed (exists) at the temporal extrema of the interval.
+        '''
+
         # only include segs in the boundary if they occur exactly once
         # otherwise its triangles converging on each other.  Tris converging on hull
         # segs have already been removed
@@ -635,6 +771,8 @@ def checkBoundaryExistence( tris ):
 
 def createMotionPlan( r1, r1Hull, r1ConCycles, r1PoiToCycLabelDict, r2, r2Hull, r2ConCycles, r2PoiToCycLabelDict, startTime, endTime ):
         '''
+        Used in the interpolateRegions function to actuallt create trinagles.
+        
         create motion triangles from r1 to r2.  start time must be earlier than end time.
         r1 must be the earlier (source) region, r2 the later (destination) region
         
